Ext.namespace('dSS', 'dSS.data');

dSS.data.DeviceStore = Ext.extend(Ext.data.Store, {
    constructor: function(config) {
        // create a record constructor for device records
        var deviceRecord = Ext.data.Record.create([
        {
            name: "name"
        },
        {
            name: "dsuid",
            mapping: 'dSUID'
        },
        {
            name: "dsid",
            mapping: 'id'
        },
        {
            name: "DisplayID"
        },
        {
            name: "blink",
            defaultValue: false
        },
        {
            name: "meterDSUID"
        },
        {
            name: "meterName"
        },
        {
            name: "zoneId",
            mapping: "zoneID"

        },
        {
            name: "zoneName"
        },
        {
            name: "isPresent"
        },
        {
            name: "isValid"
        },
        {
            name: "firstSeen"
        },
        {
            name: "inactiveSince",
            type: "date"
        },
        {
            name: "locked"
        },
        {
            name: "functionID"
        },
        {
            name: "comment"
        },
        {
            name: "productRevision"
        },
        {
            name: "productID"
        },
        {
            name: "HW",
            mapping: "hwInfo"
        },
        {
            name: "firmwareVersion"
        },
        {
            name: "buttonID"
        },
        {
            name: "buttonInputMode"
        },
        {
            name: "buttonInputCount"
        },
        {
            name: "buttonInputIndex"
        },
        {
            name: "buttonActiveGroup"
        },
        {
            name: "binaryInputs"
        },
        {
            name: "sensorInputs"
        },
        {
            name: "groups"
        },
        {
            name: "outputMode"
        },
        {
            name: "OemEanNumber"
        },
        {
            name: "OemSerialNumber"
        },
        {
            name: "OemStatus"
        },
        {
            name: "AKMInputProperty"
        },
        {
            name: "configurationLocked"
        },
        {
            name: "sensorDataValid"
        },
        {
            name: "sensorInputCount"
        },
        {
            name: "sensors",
            defaultValue: []
        },
        {
            name: "binaryInputCount"
        },
        {
            name: "assignedSensorTypes",
            defaultValue: []
        },
        {
            name: "isVdcDevice"
        },
        {
            name: "VdcConfigURL"
        },
        {
            name: "VdcHardwareModelGuid"
        },
        {
            name: "VdcModelUID"
        },
        {
            name: "VdcModelVersion"
        },
        {
            name: "VdcHardwareInfo"
        },
        {
            name: "dSUIDIndex"
        },
        {
            name: "clusterLock",
            defaultValue: false
        },
        {
            name: "lockedApartmentScenes",
            defaultValue: []
        },
        {
            name: "areas",
            defaultValue: []
        },
        {
            name: "pairedDevices",
            defaultValue: []
        },
        {
            name: "vdcOpStates",
            defaultValue: []
        }
        ]);

        // a json reader to read the device data
        var deviceReader = new Ext.data.JsonReader({
            root: "devices",
            idProperty: "dSUID"
        },
        deviceRecord);

        Ext.apply(
        this, {
            reader: deviceReader,
            sortInfo: {
                field: 'firstSeen',
                direction: 'DESC' // or 'DESC' (case sensitive for local sorting)
            }
        });

        dSS.data.DeviceStore.superclass.constructor.call(this, arguments);

        dSS.MessageBus.on('reloadsensorassignments',
                          this.getSensorAssignments.createDelegate(this), this);
        dSS.MessageBus.on('reloadbinaryinputs',
                          this.getBinaryInputs.createDelegate(this), this);
        dSS.MessageBus.on('reloadvdcopstates',
                          this.getVdcOpStates.createDelegate(this), this);

        this.on('load', function(store, records, options) {
            Ext.each(records, function(record) {
                var isVdcDevice = record.get('isVdcDevice');
                if (isVdcDevice) {
                    var VdcModelVersion = record.get('VdcModelVersion');
                    record.set('firmwareVersion', VdcModelVersion);
                } else {
                    var rev = record.get('productRevision');
                    if (rev != 0) {
                        var major = rev >> 8;
                        var mid = (rev & 0x00f0) >> 4;
                        var last = rev & 0x000f;

                        record.set('firmwareVersion', dSS.util.dec2hex(major) +
                            '.' + dSS.util.dec2hex(mid) +
                            '.' + dSS.util.dec2hex(last));
                    }
                }

                // #2232 HACK START
                var groups = record.get('groups');
                if (groups) {
                    var newgrp = [];
                    for (var i = 0; i < groups.length; i++) {
                        newgrp.push(parseInt(groups[i], 10));
                    }
                    record.set('groups', newgrp);
                }
                // #2232 HACK END

                record.commit();
             });

             var genCSS = function(record) {
                var css = "";
                var dSUID = record.get('dsuid');
                var cls = dSS.util.parseFunctionID(record.get('functionID'));
                if (cls[0] === dSS.config.deviceClass.SW) {
                    // generate CSS or each device class to work around caching
                    var g;
                    for (g = 0; g <= 10; g++) {
                        css += '.gridIcon' + dSUID + 'joker' + g +
                            ' { background:url(/icons/getDeviceIcon?dsuid=' +
                            dSUID + '&_dc=' + Math.random(1) +
                            ') center top no-repeat !important; }\n';
                        css += '.titleIcon' + dSUID + 'joker' + g +
                            ' { background-image:url(/icons/getDeviceIcon?dsuid=' +
                            dSUID + '&_dc=' + Math.random(1) +
                            ') !important; }\n';
                    }
                } else {
                    css += '.gridIcon' + dSS.util.genCSSName(record) +
                              ' { background:url(/icons/getDeviceIcon?dsuid=' +
                              dSUID + '&_dc=' + Math.random(1) +
                              ') center top no-repeat !important; }\n';
                    css += '.titleIcon' + dSS.util.genCSSName(record) +
                          ' { background-image:url(/icons/getDeviceIcon?dsuid=' +
                           dSUID + '&_dc=' + Math.random(1) +
                           ') !important; }\n';
                }
                return css;
            };

            Ext.Ajax.request({
                url: '/json/property/query',
                disableCaching: true,
                method: 'GET',
                scope: this,
                params: {
                    query: '/apartment/zones/zone0/devices/*(dSUID,comment)'
                },
                success: function(result, request) {
                    try {
                        var json = Ext.util.JSON.decode(result.responseText);
                        if (json.ok) {
                            var modified = false;

                            var css = "";
                            Ext.each(json.result.devices, function(val) {
                                var rec = store.getById(val.dSUID);
                                if (rec !== undefined) {
                                    if (val.comment !== undefined) {
                                        rec.set("comment", val.comment);
                                        modified = true;
                                    }
                                    css += genCSS(rec);
                                }
                            });
                            dSS.util.setCSS(css);
                            if (modified === true) {
                                store.commitChanges();
                            }
                        }
                    } catch (err) {
                        Ext.Msg.alert(dSS._("Error"),
                                      dSS._("Could not get comments from dSS") +
                                        ": " + err);
                    }
                    this.getSensorAssignments(store);
            },
            failure: function(result, request) {
                this.getSensorAssignments(store);
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Failure when requesting comments from dSS"));
            }
        });

        },
        this, { buffer: 1500 });
    },
    getSensorAssignments: function(store) {
        if (!store) {
            store = dSS.data.Loader.getDeviceStore();
        }

        var zoneStore = dSS.data.Loader.getZoneStore();

        Ext.Ajax.request({
            url: '/json/apartment/getAssignedSensors',
            disableCaching: true,
            method: 'GET',
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        return;
                    }
                    Ext.each(json.result.zones, function(val) {
                        var i;
                        if (val.sensors.length > 0) {
                            var zone = zoneStore.getById(val.id);
                            if (zone) {
                                zone.set('sensors', val.sensors.slice(0));
                                zone.commit();
                            }
                        }
                        for (i = 0; i < val.sensors.length; i++) {
                            var sensor = val.sensors[i];
                            var rec = store.getById(sensor.dsuid);
                            if (rec) {
                                var st = rec.get('assignedSensorTypes').slice(0);
                                if (st.indexOf(sensor.sensorType) === -1) {
                                    st.push(sensor.sensorType);
                                    rec.set('assignedSensorTypes', st);
                                    rec.commit();
                                }
                            }
                        }
                    });
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not get sensor assignments" +
                                  " from dSS") + ": " + err);
                }
                this.getVdcOpStates(store);
            },
            failure: function() {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Failure when requesting sensor " +
                              "assignments from dSS"));
                this.getVdcOpStates(store);
            }
        });
    },

    getBinaryInputs: function(store) {
        if (!store) {
            store = dSS.data.Loader.getDeviceStore();
        }

        Ext.Ajax.request({
            url: '/json/apartment/getDeviceBinaryInputs',
            disableCaching: true,
            method: 'GET',
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        return;
                    }
                    var modified = false;
                    Ext.each(json.result.devices, function(val) {
                        var device = store.getById(val.dsuid);
                        if (!device) {
                            return true; // go on
                        }

                        device.set('binaryInputs', val.binaryInputs);
                        modified = true;
                    });
                    if (modified) {
                        store.commitChanges();
                    }
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not get binary inputs" +
                                  " from dSS") + ": " + err);
                }
            },
            failure: function() {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Failure when requesting binary inputs " +
                              " from dSS"));
            }
        });
    },

    getVdcOpStates: function(store) {
        if (!store) {
            store = dSS.data.Loader.getDeviceStore();
        }

        Ext.Ajax.request({
            url: '/json/apartment/getDeviceInfo',
            disableCaching: true,
            method: 'GET',
            params: { filter: "operational", lang: dSS.config.language },
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        return;
                    }
                    var modified = false;
                    Ext.each(json.result.devices, function(val) {
                        var device = store.getById(val.dSUID);
                        if (!device) {
                            return true; // go on
                        }

                        // account for unlucky dSS API JSON
                        var states = [];
                        var key;
                        var obj = {};
                        if (val.operational) {
                            obj = val.operational.states;
                        }
                        for (key in obj) {
                            if(obj.hasOwnProperty(key)) {
                                states.push(obj[key]);
                            }
                        }

                        device.set('vdcOpStates', states);
                        modified = true;
                    });
                    if (modified) {
                        store.commitChanges();
                    }
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not get vDC operational states" +
                                  " from dSS") + ": " + err);
                }
            },
            failure: function() {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Failure when requesting vDC operational " +
                              "states from dSS"));
            }
        });
    }
});

