Ext.namespace('dSS', 'dSS.grid', "dSS.gthelper");

dSS.grid.SensorPanel = Ext.extend(Ext.grid.GridPanel, {
    constructor: function(config) {

        this.standalone = false;

        if (config.standalone === true) {
            this.standalone = true;
        }

        this.zoneAssignment = true;
        if (config.zoneAssignment !== undefined) {
            this.zoneAssignment = config.zoneAssignment;
        }

        var SensorValueRenderer = function(value, metaData, record, rowIndex,
                                           colIndex, store) {
            if (record.get('valid') === false) {
                metaData.attr = 'style="color:red;"';
            }

            var d = Date.parseDate(record.get('timestamp'),
                                   "Y-m-d H:i:s", true);

            if (!d || d.getFullYear() == 1970) {
                return "-";
            }

            var x = Math.round (value * 10) / 10;
            return x;
        };

        var SensorTypeRenderer = function(value, metaData, record, rowIndex,
                                          colIndex, store) {
            var sensor = parseInt(record.get('type'));
            if (sensor === NaN) {
                sensor = 255; // unknown type
            }

            var type = dSS.config.SensorTypeStrings[sensor];
            if (type === undefined) {
                type = record.get('type');
            } else {
                type = dSS._(type);
            }

            return type;
        };

        var defaultDateRenderer = Ext.util.Format.dateRenderer('Y-m-d\\&\\n\\b\\s\\p\\;H:i:s');

        var SensorValueTimestampRenderer = function(value, metaData, record,
                                                    rowIndex, colIndex, store) {
            if (record.get('valid') === false) {
                metaData.attr = 'style="color:red;"';
            }

            var d = Date.parseDate(record.get('timestamp'),
                                   "Y-m-d H:i:s", true);

            if (!d || d.getFullYear() == 1970) {
                return "-";
            } else {
                return defaultDateRenderer(d);
            }
        };

        var PollIntervalRenderer = function(value, metaData, record, rowIndex,
                                            colIndex, store) {
            var pollInterval = parseInt(record.get('pollinterval'));
            if (pollInterval === 0) {
                return "&infin;"
            }

            return value;
        };

        var SensorSourceRenderer = function(value, metaData, record, rowIndex,
                                            colIndex, store) {
            var type = record.get('type');
            var store = dSS.data.Loader.getDeviceStore();
            var rec = store.getById(record.get('dsuid'));
            if (rec) {
                var types = rec.get('assignedSensorTypes');
                if (types.indexOf(type) !== -1) {
                    return "&#x2713;";
                }
            }

            return "";
        };

        var columns =
        [
            {
                header: dSS._("Index"),
                width: 35,
                dataIndex: 'index'
            },
            {
                header: dSS._("Sensor type"),
                dataIndex: 'type',
                renderer: SensorTypeRenderer
            },
            {
                header: dSS._("Value"),
                dataIndex: 'value',
                width: 40,
                renderer: SensorValueRenderer
            },
            {
                header: dSS._("Value timestamp"),
                dataIndex: 'timestamp',
                renderer: SensorValueTimestampRenderer
            }
        ];

        if (!this.standalone) {
            columns.push(
            {
                header: dSS._("Poll interval"),
                dataIndex: 'pollinterval',
                width: 60,
                renderer: PollIntervalRenderer
            });
        }

        columns.push(
        {
            header: dSS._("Room sensor"),
            width: 50,
            align: 'center',
            renderer: SensorSourceRenderer
        });

        var SensorRecord = Ext.data.Record.create(
        [
            { name: 'dsuid' },
            { name: 'index' },
            { name: 'type' },
            { name: 'value' },
            { name: 'timestamp' },
            { name: 'valid' },
            { name: 'pollinterval' }
        ]);

        var SensorReader = new Ext.data.JsonReader(
            {
                root: 'sensorInputs',
                idProperty: 'index'
            },
            SensorRecord
        );

        Ext.apply(this, {
            title: dSS._("Sensor", "Sensors", 2),
            tbar: this.buildTopToolbar(),
            store: new Ext.data.Store({
                reader: SensorReader,
                sortInfo: {
                    field: 'index',
                    direction: 'ASC'
                }
            }),
            columns: columns,
            stripeRows: true,
            selModel: new Ext.grid.RowSelectionModel({ singleSelect: true }),
            viewConfig: {
                forceFit: true
            }
        });

        dSS.grid.SensorPanel.superclass.constructor.apply(this, arguments);

        if (!this.standalone) {
            dSS.MessageBus.on('loadsensordata',
                    this.loadSensorData.createDelegate(this), this);
            dSS.MessageBus.on('clearsensordata',
                    this.clearSensorData.createDelegate(this), this);
        }

        if (this.standalone) {
            this.on('afterrender',
                this.loadSensorData.createDelegate(this, [ config.dsuid ]),
                this);
        }

        this.on('rowdblclick', this.updateSensor.createDelegate(this), this);

        this.on('rowcontextmenu', this.handleContextMenu.createDelegate(this),
                this);
        this.getSelectionModel().on('rowselect',
                this.handleSelection.createDelegate(this), this);
        this.getSelectionModel().on('rowdeselect',
                this.controlToolbar.createDelegate(this, [ false ], false),
                this);
    },
    updateModel: function(dsuid, sensors) {
        var i;
        var j;
        var rec = dSS.data.Loader.getDeviceStore().getById(dsuid);
        if (!rec) {
            return;
        }

        var origsensors = rec.get('sensors').splice(0);
        var dirty = false;

        for (i = 0; i < sensors.length; i++) {
            var sensor = sensors[i];
            for (j = 0; j < origsensors.length; j++) {
                var origsensor = origsensors[j];
                if (sensor.type == origsensor.type) {
                    origsensors[j] = sensor;
                    dirty = true;
                    // not breaking out, multiple sensors of same type are
                    // not forbidden
                }
            }
        }

        if (dirty) {
            rec.set('sensors', origsensors);
            rec.commit();
        }
    },
    loadSensorData: function(dsuid) {
        if (this.dsuid !== dsuid) {
            this.lastSelectedIndex = -1;
            this.getSelectionModel().clearSelections();
        }

        this.dsuid = dsuid;
        Ext.Ajax.request({
            url: '/json/property/query',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                query: '/apartment/zones/zone0/devices/' + dsuid +
                       '/sensorInputs/*(type,index,value,valid,timestamp,pollinterval)',
            },
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (json.ok) {
                        var i;
                        for (i = 0; i < json.result.sensorInputs.length; i++) {
                            json.result.sensorInputs[i].dsuid = dsuid;
                        }
                        this.updateModel(dsuid, json.result.sensorInputs);
                        this.getStore().loadData(json.result);
                        this.getStore().filter({
                            fn : function(record) {
                                var t = record.get('type');
                                return dSS.util.isVisibleSensorType(t,
                                                            this.standalone);
                            },
                            scope: this
                        });
                    }
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not load device sensor " +
                                        "information from dSS") + ": " + err);
                }
                this.handleSelection(this.getSelectionModel());
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                          dSS._("Failure when requesting device sensor " +
                                "information from dSS"));
                this.handleSelection(this.getSelectionModel());
            }
        });
    },
    clearSensorData: function() {
        this.getStore().removeAll();
    },
    updateSensor: function(grid, rowIndex, e) {
        this.el.mask(dSS._("Requesting sensor data") + "...", 'x-mask-loading');

        if (rowIndex === undefined) {
            var sel = this.getSelectionModel();
            var record = sel.getSelected();
            rowIndex = record.get('index');
        } else {
            var sensorIndex = this.getStore().getAt(rowIndex);
            rowIndex = sensorIndex.get('index');
        }

        Ext.Ajax.request({
            url: '/json/device/getSensorValue',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                dsuid: this.dsuid,
                sensorIndex: rowIndex
            },
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (json.ok) {
                        this.loadSensorData(this.dsuid);
                    }
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not get device sensor " +
                                        "data from dSS") + ": " + err);
                }
                this.el.unmask();
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                          dSS._("Failure when requesting device sensor " +
                                "data from dSS"));
                this.el.unmask();
            }
        });
    },
    setAsZoneSensor: function(grid, rowIndex, e) {
        var sel = this.getSelectionModel();
        var record = sel.getSelected();
        var sensorType = record.get('type');

        var devStore = dSS.data.Loader.getDeviceStore();
        var devRec = devStore.getById(this.dsuid);
        if (!devRec) {
            Ext.Msg.alert(dSS._("Error"),
                          dSS.gt.strargs(
                              dSS._("Could not find device with ID %1"),
                              [
                                '&quot;' +
                                record.get("DisplayID") +
                                '&quot;'
                              ]
                          ));
            return;
        }

        this.el.mask(dSS._("Assigning sensor") + "...", 'x-mask-loading');

        var zoneID = devRec.get('zoneId');

        var zoneRec = dSS.data.Loader.getZoneStore().getById(zoneID);
        var zoneSensors = zoneRec.get('sensors');

        Ext.Ajax.request({
            url: '/json/zone/setSensorSource',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                dsuid: this.dsuid,
                sensorType: sensorType,
                id: zoneID
            },
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (json.ok) {
                        var i;
                        for (i = 0; i < zoneSensors.length; i++) {
                            var zs = zoneSensors[i];
                            if (zs.sensorType === sensorType) {
                                var oldDev = devStore.getById(zs.dsuid);
                                if (oldDev) {
                                    var oldSensors = oldDev.get('assignedSensorTypes').slice(0);
                                    var index = oldSensors.indexOf(sensorType);
                                    if (index > -1) {
                                        oldSensors.splice(index, 1);
                                    }
                                    oldDev.set('assignedSensorTypes', oldSensors);
                                    oldDev.commit();
                                }
                            }
                        }
                        this.loadSensorData.defer(1000, this, [ this.dsuid ]);
                    }
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS.gt.strargs(
                                      dSS._("Could not assign sensor %1 to " +
                                            "selected zone"),
                                      [
                                          '&quot;' +
                                          dSS.util.getNameOrDisplayID(
                                              devRec.get('name'),
                                              devRec.get('DisplayID')) +
                                          '&quot;'
                                      ]
                                  ) + ": " + err);
                }
                this.el.unmask();
                dSS.MessageBus.fireEvent('reloadsensorassignments');
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS.gt.strargs(
                                  dSS._("Failure when assigning %1 to " +
                                        "selected zone"),
                                  [
                                      '&quot;' +
                                      dSS.util.getNameOrDisplayID(
                                          devRec.get('name'),
                                          devRec.get('DisplayID')) +
                                      '&quot;'
                                  ]
                               ));

                this.el.unmask();
                dSS.MessageBus.fireEvent('reloadsensorassignments');
            }
        });

    },
    handleContextMenu: function(grid, rowIndex, e) {
        e.preventDefault();
        e.stopEvent();
        if (!this.contextMenu) {
            this.contextMenu = new Ext.menu.Menu({});
        } else {
            this.contextMenu.removeAll();
        }

        var sel = this.getSelectionModel();

        if (!sel.hasSelection()) {
            sel.selectRow(rowIndex, false);
        }

        var record = sel.getSelected();
        var sensorType = record.get('type');


        var menuItem;

        menuItem = new Ext.menu.Item(
        {
            iconCls: 'reloadAction',
            text: dSS._("Refresh sensor data"),
            handler: this.updateSensor.createDelegate(this, [this, rowIndex, e],
                                                      false)
        });
        this.contextMenu.add(menuItem);

        if (dSS.util.isAllowedSensorType(sensorType)) {
            menuItem = new Ext.menu.Item(
            {
                iconCls: 'setZoneSensorAssignment',
                text: dSS._("Set as zone sensor"),
                handler: this.setAsZoneSensor.createDelegate(this,
                                                          [this, rowIndex, e],
                                                             false)
            });
            this.contextMenu.add(menuItem);
        }

        var xy = e.getXY();
        this.contextMenu.showAt(xy);
    },
    buildTopToolbar: function() {
        var toolbar =
        [
            {
                iconCls: 'reloadAction',
                tooltip: dSS._("Refresh sensor data"),
                scope: this,
                disabled: true,
                handler: this.updateSensor.createDelegate(this, [], false)
            }
        ];

        if (this.zoneAssignment) {
            toolbar.push(
            {
                iconCls: 'setZoneSensorAssignment',
                tooltip: dSS._("Set as zone sensor"),
                scope: this,
                ref: 'assignButton',
                disabled: true,
                handler: this.setAsZoneSensor.createDelegate(this, [], false)
            });
        }
        return toolbar;
    },
    controlToolbar: function(enable, sensorType) {
        this.getTopToolbar().setDisabled(!enable);
        if (!this.zoneAssignment) {
            return;
        }

        if (enable) {
            if (dSS.util.isAllowedSensorType(sensorType)) {
                this.getTopToolbar().assignButton.setDisabled(false);
            } else {
                this.getTopToolbar().assignButton.setDisabled(true);
            }
        }
    },
    handleSelection: function(selModel) {
        if (selModel.hasSelection()) {
            var record = selModel.getSelected();
            var index = this.getStore().indexOf(record);
            this.lastSelectedIndex = index;
            this.controlToolbar(true, record.get('type'));
        } else {
            // for some reason automatic selection produces Ext errors when
            // the grid is used inside our dialog
            if (this.standalone) {
                this.controlToolbar(false);
                return;
            }

            if (this.lastSelectedIndex != -1) {
                this.getView().focusRow(this.lastSelectedIndex);
                var record = this.getStore().getAt(this.lastSelectedIndex);
                if (record !== undefined) {
                    selModel.selectRecords([record], false);
                    this.controlToolbar(true, record.get('type'));
                }
            } else {
                this.controlToolbar(false);
            }
        }
    }
});

Ext.reg('dsssensorpanel', dSS.grid.SensorPanel);
