/*
 * jPicker plugin integration with Ext JS 3.3.4
 *
 * Note: this user extention assumes a modified version of jPicker.

 * Copyright (c) 2016, digitalSTROM AG, Zurich, Switzerland
 * Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>
 *
 * This user extention is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This user extention is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this user extention. If not, see <http://www.gnu.org/licenses/>.
 *
 */

Ext.ns('Ext.ux');

Ext.ux.JPicker = Ext.extend(Ext.Panel, {

    /**
     * @cfg {String}
     * Path to Jpicker images, defaults to ./jpicker/images/
     */
    clientPath: 'js/extjs/ux/images/',

    /**
     * @cfg {Object}
     * Initial picker color in HSV format.
     */
    initialColor: { h: 0, s: 0, v: 0 },

	/**
     * @cfg {Object}
     * Localization object that is passed to jPicker.
     */
    localization: undefined,

    initComponent: function() {
        this.addEvents('jpickercolorchanged'); // args: this panel, color
        Ext.applyIf(this,{});
        
        this.oldColor = this.initialColor;

        Ext.ux.JPicker.superclass.initComponent.call(this);
       
        this.on('afterrender', function(panel) {
            panel.oldColor = panel.initialColor;

            var cchanged = function() {
                var color = $.jPicker.List[0].color.active.val('hsv');
                if (!panel.equalHSVColors(color, panel.oldColor)) {
                    panel.fireEvent('jpickercolorchanged', panel, color);
                }
            };

            var options =
            {
                color: {
                    active: new $.jPicker.Color(this.initialColor)
                },
                images: {
                    clientPath: this.clientPath
                }
            };

            if (panel.localization !== undefined) {
                options.localization = panel.localization;
            }

            $(panel.body.dom).jPicker(options, null, null, null, cchanged);
        },
        this);
    },
    equalHSVColors: function(oldcol, newcol) {
        if ((oldcol.h === newcol.h) &&
            (oldcol.s === newcol.s) &&
            (oldcol.v === newcol.v)) {
            return true;
        }
        return false;
    },
    setColor: function(hsv) {
        if ($.jPicker.List.length > 0) {
            $.jPicker.List[0].color.active.val('hsv', hsv);
        }
    },
    getColor: function() {
        if ($.jPicker.List.length > 0) {
            return $.jPicker.List[0].color.active.val('hsv');
        } else {
            return this.initialColor;
        }
    }
});

Ext.reg('jpickerpanel', Ext.ux.JPicker);
